// v3.8.1
// Tested for v3.8
// Contributors: Nick Velich

// Cancel Incident v3.8.1 -- Hide Task for Analysts and if the Incident is already Resolved
app.custom.formTasks.add('Incident', null, function (formObj, viewModel) {
	formObj.boundReady(function(){
	var vm = pageForm.viewModel;
	//If the Service Request is Completed don't show the Add MA task
		if (vm.Status.Name == "Resolved"){
			$( ".taskmenu li:contains('Cancel Incident')" ).hide() 
		}
		//If the user is not an analyst, hide the task
		if (session.user.Analyst) { 
			$( ".taskmenu li:contains('Cancel Incident')" ).hide() 
		} 
	});
	return;
});

// Cancel Incident v3.8.1 -- Add tasks to tasks pane
app.custom.formTasks.add('Incident', "Cancel Incident", function (formObj, viewModel) {	
	CIR_loadResolutionDescription();
});

// Cancel Incident v3.8.1 -- Load Resolution Description window and allow user to enter a reason for cancellation
function CIR_loadResolutionDescription (){
	//use requirejs to load the HTML template first
	require(["text!/CustomSpace/customtasks.ircancel.html"], function (template) {
		//make a jQuery obj
		cont = $(template);

		//create a view model to handle the UX
		var _vmWindow = new kendo.observable({	
			comment: "",
            okEnabled: false,
            charactersRemaining: "4000",
            textCounter: function () {
                var maximumLength = 4000;
                var val = this.comment.length;
                (val > maximumLength) ? this.comment.substring(0, maximumLength) : this.set("charactersRemaining", maximumLength - val);
                (val > 0) ? this.set("okEnabled", true) : this.set("okEnabled", false);
            },
			okClick: function () {
				var description = this.get("comment");

				//They clicked OK now call the Resolve Incident function and send the Resolution Category ID that is selected
				CIR_processSubmission(description);
				customWindow.close();				
			},
			cancelClick: function () {
				customWindow.close();
			}
		});
	 
		//create the kendo window
		customWindow = cont.kendoWindow({
			title: "Cancel Incident",
			resizable: false,
			modal: true,
			viewable: false,
			width: 500,
			height: 275,
			close: function () { 
			},
			activate: function () {
				//on window activate bind the view model to the loaded template content
				kendo.bind(cont, _vmWindow);
			}
		}).data("kendoWindow");
	 
		//now open the window
		customWindow.open().center();
	});
}

// Cancel Incident v3.8.1 -- Process the user's submission by creating the current/final ir objects, calling the resolve function, and committing/applying mask
function CIR_processSubmission(description) {
	var ogir = pageForm.viewModel;
	
	var finalir = JSON.parse(JSON.stringify(ogir));
	var irid = ogir.Id;
	
	CIR_resolveIncident(finalir,description);
	
	//Stringify data and commit changes 
	var strData = { "formJson":{"original": ogir, "current": finalir}}
		$.ajax({
			url: "/api/V3/Projection/Commit",
			type: "POST",
			contentType: "application/json; charset=utf-8",
			dataType: "json",
			data: JSON.stringify(strData) ,
			success: function (data2) {
				app.lib.mask.apply('Cancelling ' + finalir.Id);			
				window.location.href = window.location.protocol + "//" + window.location.hostname + "/Incident/Edit/" + irid
			}
		}); 
}

// Cancel Incident v3.8.1 -- Cancel the IR using the user-input description. Set the resolution time, resolved by user, status, and create an
// action log entry indicating this was resolved, with the user's description for better visibility.
function CIR_resolveIncident (finalir,description) {
	var incident = finalir;
	
	incident.Status.Id = "2b8830b6-59f0-f574-9c2a-f4b4682f1681"; // Resolved, as Cancelled does not exist for IRs
	incident.ResolutionDescription=description;
	incident.ResolutionCategory.Id = "4ddea88e-2da7-8f16-001a-af68db9a6b8e"; // Best to use a custom Resolution Category for Cancelled
	
	var date = new Date()
	var addminute = 60*1000
	date.setTime(date.getTime() + addminute);
	
	var uid = session.user.Id;
	var displayname = session.user.Name;
	
	incident.ResolvedDate = date;
	
	// Action log comment/icon indicating resolution
	incident.AppliesToWorkItem = [{
		"ActionType": {
		  "Id": "5ca2cfee-6740-1576-540B-ce17222840b8",
          "Name": "Record Resolved"
			},	
		"Description": description,
		"EnteredBy": displayname,
		"EnteredDate": new Date().toISOString().split(".")[0],
		"LastModified": new Date().toISOString().split(".")[0],
		"Title": "Record Resolved",
        "Image": "/Content/Images/Icons/ActionLogIcons/recordresolved.png",
        "BillableTime": {
          "BasedId": null,
          "DisplayName": null
			},
        "LastUpdatedDisplay": null		
		}];
	
	// ResolvedBy relationship
	incident.RelatesToTroubleTicket = [{
		"ClassTypeId": "eca3c52a-f273-5cdc-f165-3eb95a2b26cf",
		"BaseId": uid,
		"DisplayName": displayname
	}];	
}
